/*
 * Gravity.java
 * Copyright (C) 1998 ymnk, JCraft Inc.
 *
 * This file is part of JSpringies, a mass and spring simulation system for Java
 *
 * JSpringies is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * JSpringies is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XSpringies; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package com.jcraft.jspringies;

class Gravity extends Force{
  final static double PI = 3.14159265358979323846;
  final static double radian = PI / 180.0;
  double gx, gy;
  Gravity(Space s, double v, double m){
    super(s, v, m);
    update();
  }
  void setVal(double v){
    super.setVal(v);
    update();
  }
  void setMisc(double m){
    super.setMisc(m);
    update();
  }
  private void update(){
    gx = gval * Math.sin(gmisc * radian);
    gy = gval * Math.cos(gmisc * radian);
  }

  void force(){
    Mass m;
    for(int i=0; i<sp.masses.length; i++){
      m = sp.masses[i];
      if (m.AliveNotFixedp()) {
	m.ax += gx;
	m.ay += gy;
      }
    }
  }
}
