/*
 * JSpringies.java
 * Copyright (C) 1998 ymnk, JCraft Inc.
 *
 * This file is part of JSpringies, a mass and spring simulation system for Java
 *
 * JSpringies is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * JSpringies is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XSpringies; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package com.jcraft.jspringies;

import java.awt.* ;
import java.awt.image.* ;
import java.applet.*;
import java.io.* ;
import java.net.* ;

public class JSpringies extends java.applet.Applet implements Runnable{
  static Thread thread=null;

  TextArea credit=null;
  static String credits=null;
  static {
    String[] foo ={
"JSpringies 1.0.1, by ymnk. \n",
"With thanks to Douglas DeCarlo, \n",
"  the author of XSpringies, \n",
"  http://www.cs.rutgers.edu/~decarlo .\n",
"Comments: jspringies@jcraft.com ."};

    StringBuffer strb=new StringBuffer();
    for(int i=0; i<foo.length; i++){
      strb.append(foo[i]);
    }
    credits=strb.toString();
  }

  Space sp = null;
  ControlPanel controlpanel = null;

  int width=640;
  int height=548;

  Image offi=null;
  Graphics offg=null;

  boolean csp=false; // colorful spring
  String initialdata=null;

  public void init(){
    {
      String s;
      s=getParameter("width");
      if(s!=null){ width=Integer.parseInt(s); }
      s=getParameter("height");
      if(s!=null){ height=Integer.parseInt(s); }
      s=getParameter("init");
      if(s!=null){ initialdata=s; }
      s=getParameter("credit");
      if(s!=null){
	credit=new TextArea(credits, 6, 35, TextArea.SCROLLBARS_NONE);
	credit.setForeground(Color.white);
	credit.setBackground(Color.darkGray);
	credit.setEditable(false);
      }
    }

    sp = new Space((double)width, (double)height);

    offi=createImage(width, height);
    offg=offi.getGraphics();

    controlpanel = new ControlPanel(this);

    if(credit!=null){
      add(credit);
    }

    layout();
    show();

    if(initialdata!=null){
      LoadData ld=new LoadData();
      ld.load(getDocumentBase(), initialdata, sp);
      controlpanel.syncState();
    }

    repaint();
  }
  synchronized Thread kick() {
    if(thread == null){
      if(credit!=null){
	credit.hide();
	credit=null; // go away
      }
      thread=new Thread(this);
      thread.start();
    }
    return thread;
  }

  public void update(Graphics g) { paint(g); }

  synchronized public void stop(){
    if(thread!=null){
      thread.stop();
      thread=null;
      controlpanel.sButton.setLabel("Start");
    }
  }

  public void run(){
    while(thread != null) {
      try {
//      Thread.yield();
//      Thread.sleep((int)( 1 ));
	Thread.sleep((int)( 5 ));
      } 
      catch (Exception e){}
      repaint();
    }
  }

//  public boolean mouseDown(java.awt.Event evt, int x, int y) { return true; }
//  public boolean mouseUp(java.awt.Event evt, int x, int y) { return true; }
//  public boolean mouseDrag(java.awt.Event evt, int x, int y) {return true; }

  void draw(Graphics g){
    g.setColor(Color.darkGray);
    g.fillRect(0, 0, width, height);
    g.setColor(Color.white);


    if(thread==null){
    }

    if(sp.masses==null && sp.springs==null){
      return;
    }

    if(sp.masses!=null){
      Mass mass;
      for(int i=0; i<sp.masses.length; i++){
	mass = sp.masses[i];
	g.drawOval((int)(mass.x-mass.radius/2.0), 
		   (int)(mass.y-mass.radius/2.0), 
		   (int)(mass.radius), (int)(mass.radius));
      }
    }

    if(sp.springs!=null){
      Mass m1, m2;
      Spring spring;

      double foo;
      double dx, dy;

      for(int i=0; i<sp.springs.length; i++){
	spring=sp.springs[i];
	m1=spring.mass1; m2=spring.mass2;

	if(csp){
	  dx=m1.x-m2.x; dy=m1.y-m2.y; 
	  foo=Math.sqrt(dx * dx + dy * dy);
	  foo-=spring.restlen;
	  if(Math.abs(foo)<0.001) g.setColor(Color.white);
	  else if(foo<0.0) g.setColor(Color.blue);
	  else g.setColor(Color.red);
	}

	g.drawLine((int)m1.x, (int)(m1.y), (int)m2.x, (int)m2.y);
      }
      if(csp){
	g.setColor(Color.white);
      }
    }
  }

  public void paint(Graphics g) {
    sp.animate_obj();
    if (offg != null) { 
      draw(offg);
      g.drawImage(offi, 0, 0, this);
    } 
//    else {
//      draw(g);
//    }
  }
}

class ControlPanel extends Frame {
  JSpringies spg=null;
  public Button sButton=null;
  Checkbox cbgrv, cbmass, cbatt, cbwall, cbcsp;
  DataList f = null;

  ControlPanel(JSpringies spg) {
    this.spg=spg;
    init();
  }
  void init(){
    GridBagLayout  gridbag  =  new  GridBagLayout();
    GridBagConstraints  gbc  =  new  GridBagConstraints();

    setLayout(gridbag);
    resize(300,150);

    gbc.fill  =  GridBagConstraints.HORIZONTAL;
    gbc.weightx  =  1.0;
    gbc.gridwidth  =  GridBagConstraints.REMAINDER;

    Panel p1 = new Panel();
    p1.add((sButton=new Button("Start")));
    p1.add(new Button("Load..."));
    gridbag.setConstraints(p1, gbc);

    add(p1);  

    gbc.gridwidth  =  GridBagConstraints.REMAINDER;
    gbc.gridheight  =  1;

    cbgrv=new Checkbox("Do gravity", null, false);
    gridbag.setConstraints(cbgrv, gbc);  add(cbgrv);
    cbmass=new Checkbox("Keep center of mass in the middle force", null, false);
    gridbag.setConstraints(cbmass, gbc); add(cbmass);
    cbatt=new Checkbox("Do point attraction force", null, false);
    gridbag.setConstraints(cbatt, gbc); add(cbatt);
    cbwall=new Checkbox("Wall attract/repel force", null, false);
    gridbag.setConstraints(cbwall, gbc); add(cbwall);

    cbcsp=new Checkbox("Colorful spring, exapnsion(red)/contraction(blue)", null, false);
    gridbag.setConstraints(cbcsp, gbc); add(cbcsp);

    show();
  }

  public boolean handleEvent(Event evt) {
    switch(evt.id) {
    case Event.WINDOW_DESTROY: 
      System.out.println("Exiting...");
      System.exit(0);
      return true;
    case Event.ACTION_EVENT: 
      if(evt.target instanceof Button){
	if("Start".equals(evt.arg)) {
	  if(spg.sp.masses==null){ return true; }
	  if(spg.thread == null) {
	    sButton.setLabel("Stop");
	    spg.kick();
	  }
	}
	else if("Stop".equals(evt.arg)) {
	  if(spg.thread != null) {
	    spg.stop();
	    sButton.setLabel("Start");
	  }
	}
	else if("Load...".equals(evt.arg)) {       
	  if(f == null) { f=new DataList(spg); }
	  f.show();
	}
	return true;
      }
      else if(evt.target instanceof Checkbox){
	if(evt.target==cbgrv) {       
	  spg.sp.gravity.setEnable(cbgrv.getState());
	}
	else if(evt.target==cbmass){
	  spg.sp.cmass.setEnable(cbmass.getState());
	}
	else if(evt.target==cbatt){
	  spg.sp.pattraction.setEnable(cbatt.getState());
	}
	else if(evt.target==cbwall){
	  spg.sp.wall.setEnable(cbwall.getState());
	}
	else if(evt.target==cbcsp){
	  spg.csp=cbcsp.getState();
	  spg.repaint();
	}
	return true;
      }
    default: 
      return false;
    }
  }
  void syncState(){
    cbgrv.setState(spg.sp.gravity.getEnable());
    cbmass.setState(spg.sp.cmass.getEnable());
    cbatt.setState(spg.sp.pattraction.getEnable());
    cbwall.setState(spg.sp.wall.getEnable());
  }
}

class DataList extends Frame {
  Panel p = null;
  List l = null;
  Checkbox b = null;
  InputStream scan = null;
  JSpringies spg=null;

  DataList(JSpringies spg){
    super();
    this.spg=spg;
    setLayout(new BorderLayout());
    resize(200,180);
    p=new Panel();
    l=new List(5,false);
    b=new Checkbox("Browse");
    try {
      URL url= new URL(spg.getDocumentBase(), "lib/data");
      URLConnection urlc= url.openConnection();
      scan=urlc.getInputStream();
    } 
    catch (Exception e) {
      System.out.println(e);
      return;
    }
    String line=null;
    while (true) {
      line=readline();
      if(line==null) break;
      l.addItem(line);
    }
    l.select(0);

    p.add("Center",l);
    add("Center",p);
    p = new Panel();
    p.add(new Button("Ok"));
    p.add(new Button("Cancel"));
    p.add(b);
    add("South", p);
    Dimension d = new Dimension(size());
    layout();
    show();
  }

  public boolean handleEvent(Event evt) {
    switch(evt.id){
    case Event.ACTION_EVENT:
      if("Cancel".equals(evt.arg)) {
	hide();
	b.setState(false);
      }
      else if("Ok".equals(evt.arg)) {
	if(spg.thread!=null){
	  Toolkit.getDefaultToolkit().beep();
	  return false;
	}
	spg.sp.clear();
	if(b.getState()==false){
	  hide();
	}
	String file=l.getSelectedItem();
	if(!file.endsWith(".xsp") &&
	   !file.endsWith(".xsp.gz")){
	  return false;
	}

	LoadData ld=new LoadData();
	if(!ld.load(spg.getDocumentBase(), file, spg.sp)){
	  return false;
	}

	spg.controlpanel.syncState();

	Mass m=null;
	if(spg.sp.masses!=null){	  
	  for(int i=0; i<spg.sp.masses.length; i++){
	    m = spg.sp.masses[i];
	    m.y = spg.sp.height-m.y;
	  }
	}
	//spg.sp.cur_grav[spg.sp.FR_GRAV]=-1*spg.sp.cur_grav[spg.sp.FR_GRAV];
	if(spg.sp.center_id >= 0){
	  m=null;
	  if(spg.sp.masses!=null){	  
	    for(int i=0; i<spg.sp.masses.length; i++){
	      m = spg.sp.masses[i];
	      if(spg.sp.center_id==m.Id()){ break; }
	    }
	  }
	  spg.sp.center_mass=m;
	}
	spg.repaint();
      }
      return true;
    default: 
      return false;
    }
  }

  private String readline() {
    StringBuffer rtn=new StringBuffer();
    int temp;
    do {
      try { temp=scan.read(); }
      catch(Exception e) {
	return(null);
      }
      if (temp==-1) {
	return(null);
      }
      if (temp!=0&&temp!='\n') rtn.append((char)temp);
    } while(temp!='\n');
    return(rtn.toString());
  }
}
