/*
 * LoadData.java
 * Copyright (C) 1998 ymnk, JCraft Inc.
 *
 * This file is part of JSpringies, a mass and spring simulation system for Java
 *
 * JSpringies is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * JSpringies is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XSpringies; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package com.jcraft.jspringies;

import java.util.*;
import java.io.*;
import java.net.* ;

class LoadData {
  InputStream scan = null;
  String line,tokje;
  StringTokenizer tok;
  Space sp;

  public LoadData () {};

  public boolean load (URL base, String file, Space sp) {
    InputStream in=null;

    // loading from jar archive;
    try {
      in=LoadData.class.getResourceAsStream("lib/"+file);
    }
    catch(Exception e){
      in=null;
    }

    if(in==null){
      try {
	URL url= new URL(base, "lib/"+file);
	URLConnection urlc= url.openConnection();
	in=urlc.getInputStream();
	if(file.endsWith(".gz")){
	  in=new java.util.zip.GZIPInputStream(in);
	}
      } 
      catch (Exception e) { 
	System.out.println(e);
	return false;
      }
    }
    return load(in, sp);
  }

  public boolean load (String config_path, Space sp) {
    FileInputStream scan=null;
    try { scan = new FileInputStream(config_path); } 
    catch (Exception e) {
      System.out.println("File not Found"); 
      return false;
    }	
    return load ((InputStream)scan, sp);
  }

  public boolean load (InputStream scan, Space sp) {
    this.scan=scan;
    this.sp=sp;
    while (true) {
      // System.out.println(line);
      line=readline();
      if(line==null) break;
      tok=new StringTokenizer(line," \n\r\t");
      if (tok.hasMoreTokens()) {
	tokje=tok.nextToken();
	if (tokje.equals("mass")) {
          masscommand(sp);
          continue;
        }
	if (tokje.equals("spng")) {spngcommand(sp); continue; }
	if (tokje.equals("cmas")) {sp.cur_mass=getdouble(); continue; }
	if (tokje.equals("elas")) {sp.cur_rest=getdouble(); continue; }
	if (tokje.equals("kspr")) {sp.cur_ks=getdouble(); continue; }
	if (tokje.equals("kdmp")) {sp.cur_kd=getdouble(); continue; }
	if (tokje.equals("fixm")) {
          int foo=getint();
          sp.fix_mass=(foo==0)? false : true;
          continue;           
        }
	if (tokje.equals("shws")) {
	  int foo=getint();
          sp.show_spring=(foo==0)? false : true;
          continue;           
        }
	if (tokje.equals("cent")) { sp.center_id=getint(); continue; }
	if (tokje.equals("frce")) {
          int which=getint();
          int foo=getint();
	  Force frce=null;
	  switch(which){
 	    case 0: frce=sp.gravity; break;
 	    case 1: frce=sp.cmass; break;
 	    case 2: frce=sp.pattraction; break;
 	    case 3: frce=sp.wall; break;
	  }
	  if(frce!=null){
	    frce.setEnable(foo>0 ? true : false);
	    frce.setVal(getdouble());
	    frce.setMisc(getdouble());
	  }
	  else{
	      getdouble();
	      getdouble();
	  }
	  continue;
        }
	if (tokje.equals("visc")) { sp.cur_visc=getdouble(); continue; }
	if (tokje.equals("stck")) { sp.cur_stick=getdouble(); continue; }
	if (tokje.equals("step")) { sp.cur_dt=getdouble(); continue; }
	if (tokje.equals("prec")) { sp.cur_prec=getdouble(); continue; }
	if (tokje.equals("adpt")) {
	  int foo=getint();
          sp.adaptive_step=(foo==0)? false : true;
          continue;           
        }
	if (tokje.equals("gsnp")) {
          sp.cur_gsnap=getdouble();
	  int foo=getint();
          if(foo==0){sp.grid_snap=false;}
          else{sp.grid_snap=true;}
          continue;           
        }
	if (tokje.equals("wall")) {
	  Wall wall=sp.wall;
	  wall.setTop((getint()==0 ? false : true));
	  wall.setLeft((getint()==0 ? false : true));
	  wall.setRight((getint()==0 ? false : true));
	  wall.setBottom((getint()==0 ? false : true));
	  //System.out.println(wall);
          continue;           
        }
      }
    }
    return true;
  }
  void masscommand(Space sp) {
    int id; 
    boolean foo=false;
    double x, y, vx, vy, mass, elastic;
    id=getint();
    x=getdouble(); y=getdouble();
    vx=getdouble(); vy=getdouble();
    mass=getdouble(); elastic=getdouble();
    if(mass < 0){ foo=true; mass *= -1;}
    sp.addMass(id, x, y, vx, vy, mass, elastic);
    Mass m=null;
    if(sp.masses!=null){
      for(int i=0; i<sp.masses.length; i++){
	m = sp.masses[i];
	if(id==m.Id()){ break; }
      }
    }
    if(m==null){ return; }
    if(foo){m.Fixed();}
    m.radius = mass_radius(m.mass);
  }

  int mass_radius(double m) {
    int rad;
    rad = (int)(2 * Math.log(4.0 * m + 1.0));
    if (rad < 1) rad = 1;
    if (rad > 64) rad = 64;
    return rad + sp.spthick/2;
  }

  void spngcommand(Space sp) {
    int id, m1, m2; 
    double ks, kd, restlen;
    id=getint();  m1=getint();  m2=getint();
    kd=getdouble();  ks=getdouble();  restlen=getdouble();

    Mass mass1=null,mass2=null,mass=null;
    if(sp.masses!=null){
      for(int i=0; i<sp.masses.length; i++){
	mass = sp.masses[i];
	if(m1==mass.Id()){ mass1=mass; }
	if(m2==mass.Id()){ mass2=mass; }
	if(mass1!=null && mass2!=null ){break;}
      }
    }
    if(mass1==null||mass2==null){ return; }

    sp.addSpring(id, mass1, mass2, kd, ks, restlen);
  }
 
  String readline() {
    StringBuffer rtn=new StringBuffer();
    int temp;
    do {
      try {
        temp=scan.read();
      }
      catch(Exception e) {
        return(null);
      }
      if (temp==-1) {
	return(null);
      }
      if (temp!=0) rtn.append((char)temp);
    } while(temp!='\n');
    return(rtn.toString());
  }
  int getint(){
    tokje=tok.nextToken();
    return Integer.valueOf(tokje).intValue();
  }
  double getdouble(){
    tokje=tok.nextToken();
    return Double.valueOf(tokje).doubleValue();
  }

}
