/*
 * Wall.java
 * Copyright (C) 1998 ymnk, JCraft Inc.
 *
 * This file is part of JSpringies, a mass and spring simulation system for Java
 *
 * JSpringies is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * JSpringies is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XSpringies; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package com.jcraft.jspringies;

class Wall extends Force{
  double width, height;
  boolean top, left, right, bottom;
  Wall(Space s, double v, double m){
    super(s, v, m);
    width=s.width;
    height=s.height;
    top=left=right=bottom=true;
  }
  void setTop(boolean b){top=b;}
  void setLeft(boolean b){left=b;}
  void setRight(boolean b){right=b;}
  void setBottom(boolean b){bottom=b;}

  void force(){
    double dax, day, dist;
    Mass m;
    for(int i=0; i<sp.masses.length; i++){
      m = sp.masses[i];
      if (m.AliveNotFixedp()) {
	dax = 0; day = 0;
	if (left && (dist = m.x - m.radius) >= 0) {
	  if (dist < 1)  dist = 1;
	  dist = Math.pow(dist, gmisc);
	  dax += gval / dist;
	}
	if (right && (dist = width - m.radius - m.x) >= 0) {
	  if (dist < 1)  dist = 1;
	  dist = Math.pow(dist, gmisc);
	  dax -= gval / dist;
	}
	if (bottom && (dist = height - m.radius - m.y) >= 0) {
	  if (dist < 1)  dist = 1;
	  dist = Math.pow(dist, gmisc);
	  day -= gval / dist;
	}
	if (top && (dist = m.y - m.radius) >= 0) {
	  if (dist < 1)  dist = 1;
	  dist = Math.pow(dist, gmisc);
	  day += gval / dist;
	}
	m.ax += dax;
	m.ay += day;
      }
    }
  }
}
